import liblistenbrainz

client = liblistenbrainz.ListenBrainz()
listen = client.get_playing_now('iliekcomputers')
assert listen is not None
print("Track name:", listen.track_name)
print("Artist name:", listen.artist_name)



import liblistenbrainz

client = liblistenbrainz.ListenBrainz()
listens = client.get_listens('iliekcomputers')
for listen in listens:
    print("Track name:", listen.track_name)
    print("Artist name:", listen.artist_name)



# liblistenbrainz - A simple client library for ListenBrainz
# Copyright (C) 2020 Param Singh <iliekcomputers@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import json
import os
import liblistenbrainz
import requests
import time
import unittest
import uuid

from liblistenbrainz import errors
from unittest import mock

TEST_DATA_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'testdata')


class ListenBrainzClientTestCase(unittest.TestCase):

    def setUp(self):
        self.client = liblistenbrainz.ListenBrainz()

    @mock.patch('liblistenbrainz.client.requests.get')
    def test_get_injects_auth_token_if_available(self, mock_requests_get):
        mock_requests_get.return_value = mock.MagicMock()
        self.client._get('/1/user/iliekcomputers/listens')
        mock_requests_get.assert_called_once_with(
            'https://api.listenbrainz.org/1/user/iliekcomputers/listens',
            params={},
            headers={},
        )

        mock_requests_get.reset_mock()
        self.client.is_token_valid = mock.MagicMock(return_value=True)
        auth_token = str(uuid.uuid4())
        self.client.set_auth_token(auth_token)
        self.client._get('/1/user/iliekcomputers/listens')
        mock_requests_get.assert_called_once_with(
            'https://api.listenbrainz.org/1/user/iliekcomputers/listens',
            params={},
            headers={'Authorization': f'Token {auth_token}'}
        )


    @mock.patch('liblistenbrainz.client.requests.post')
    def test_post_injects_auth_token_if_available(self, mock_requests_post):
        mock_requests_post.return_value = mock.MagicMock()
        self.client._post('/1/user/iliekcomputers/listens')
        mock_requests_post.assert_called_once_with(
            'https://api.listenbrainz.org/1/user/iliekcomputers/listens',
            data=None,
            headers={},
        )

        mock_requests_post.reset_mock()
        self.client.is_token_valid = mock.MagicMock(return_value=True)
        auth_token = str(uuid.uuid4())
        self.client.set_auth_token(auth_token)
        self.client._post('/1/user/iliekcomputers/listens')
        mock_requests_post.assert_called_once_with(
            'https://api.listenbrainz.org/1/user/iliekcomputers/listens',
            data=None,
            headers={'Authorization': f'Token {auth_token}'}
        )


    def test_client_get_listens(self):
        self.client._get = mock.MagicMock()
        with open(os.path.join(TEST_DATA_DIR, 'get_listens_happy_path_response.json')) as f:
            response_json = json.load(f)
        self.client._get.return_value = response_json
        received_listens = self.client.get_listens('iliekcomputers')
        expected_listens = response_json['payload']['listens']
        self.client._get.assert_called_once_with(
            '/1/user/iliekcomputers/listens',
            params={},
        )
        for i in range(len(expected_listens)):
            self.assertEqual(received_listens[i].listened_at, expected_listens[i]['listened_at'])
            self.assertEqual(received_listens[i].track_name, expected_listens[i]['track_metadata']['track_name'])


    def test_client_get_listens_with_max_ts(self):
        ts = int(time.time())
        self.client._get = mock.MagicMock()
        with open(os.path.join(TEST_DATA_DIR, 'get_listens_happy_path_response.json')) as f:
            response_json = json.load(f)
        self.client._get.return_value = response_json
        received_listens = self.client.get_listens('iliekcomputers', max_ts=ts)
        self.client._get.assert_called_once_with(
            '/1/user/iliekcomputers/listens',
            params={'max_ts': ts},
        )
        expected_listens = response_json['payload']['listens']
        for i in range(len(expected_listens)):
            self.assertEqual(received_listens[i].listened_at, expected_listens[i]['listened_at'])
            self.assertEqual(received_listens[i].track_name, expected_listens[i]['track_metadata']['track_name'])


    def test_client_get_listens_with_min_ts(self):
        ts = int(time.time())
        self.client._get = mock.MagicMock()
        with open(os.path.join(TEST_DATA_DIR, 'get_listens_happy_path_response.json')) as f:
            response_json = json.load(f)
        self.client._get.return_value = response_json
        received_listens = self.client.get_listens('iliekcomputers', min_ts=ts)
        self.client._get.assert_called_once_with(
            '/1/user/iliekcomputers/listens',
            params={'min_ts': ts},
        )
        expected_listens = response_json['payload']['listens']
        for i in range(len(expected_listens)):
            self.assertEqual(received_listens[i].listened_at, expected_listens[i]['listened_at'])
            self.assertEqual(received_listens[i].track_name, expected_listens[i]['track_metadata']['track_name'])


    def test_client_get_listens_with_count(self):
        self.client._get = mock.MagicMock()
        with open(os.path.join(TEST_DATA_DIR, 'get_listens_happy_path_response.json')) as f:
            response_json = json.load(f)
        self.client._get.return_value = response_json
        received_listens = self.client.get_listens('iliekcomputers', count=50)
        self.client._get.assert_called_once_with(
            '/1/user/iliekcomputers/listens',
            params={'count': 50},
        )
        expected_listens = response_json['payload']['listens']
        for i in range(len(expected_listens)):
            self.assertEqual(received_listens[i].listened_at, expected_listens[i]['listened_at'])
            self.assertEqual(received_listens[i].track_name, expected_listens[i]['track_metadata']['track_name'])


    def test_client_get_playing_now(self):
        self.client._get = mock.MagicMock()
        with open(os.path.join(TEST_DATA_DIR, 'get_playing_now_happy_path_response.json')) as f:
            response_json = json.load(f)
        self.client._get.return_value = response_json
        received_listen = self.client.get_playing_now('iliekcomputers')
        self.client._get.assert_called_once_with(
            '/1/user/iliekcomputers/playing-now',
        )
        expected_listen = response_json['payload']['listens'][0]
        self.assertIsNotNone(received_listen)
        self.assertIsNone(received_listen.listened_at)
        self.assertEqual(received_listen.track_name, expected_listen['track_metadata']['track_name'])


    def test_client_get_playing_now_no_listen(self):
        self.client._get = mock.MagicMock()
        with open(os.path.join(TEST_DATA_DIR, 'no_playing_now.json')) as f:
            response_json = json.load(f)
        self.client._get.return_value = response_json
        received_listen = self.client.get_playing_now('iliekcomputers')
        self.client._get.assert_called_once_with(
            '/1/user/iliekcomputers/playing-now',
        )
        self.assertIsNone(received_listen)

    @mock.patch('liblistenbrainz.client.requests.post')
    @mock.patch('liblistenbrainz.client.json.dumps')
    def test_submit_single_listen(self, mock_json_dumps, mock_requests_post):
        ts = int(time.time())
        listen = liblistenbrainz.Listen(
            track_name="Fade",
            artist_name="Kanye West",
            release_name="The Life of Pablo",
            listened_at=ts,
        )

        expected_payload = {
            'listen_type': 'single',
            'payload': [
                {
                    'listened_at': ts,
                    'track_metadata': {
                        'track_name': 'Fade',
                        'artist_name': 'Kanye West',
                        'release_name': 'The Life of Pablo',
                    }
                }
            ]
        }

        mock_json_dumps.return_value = "mock data to be sent"
        mock_requests_post.return_value.json.return_value = {'status': 'ok'}
        auth_token = str(uuid.uuid4())
        self.client.is_token_valid = mock.MagicMock(return_value=True)
        self.client.set_auth_token(auth_token)
        response = self.client.submit_single_listen(listen)
        mock_requests_post.assert_called_once_with(
            'https://api.listenbrainz.org/1/submit-listens',
            data="mock data to be sent",
            headers={'Authorization': f'Token {auth_token}'}
        )
        mock_json_dumps.assert_called_once_with(expected_payload)
        self.assertEqual(response['status'], 'ok')

    def test_submit_payload_exceptions(self):
        ts = int(time.time())
        listen = liblistenbrainz.Listen(
            track_name="Fade",
            artist_name="Kanye West",
            release_name="The Life of Pablo",
            listened_at=ts,
        )

        # check that it doesn't try to submit without an auth token
        with self.assertRaises(errors.AuthTokenRequiredException):
            self.client.submit_multiple_listens([listen])

        # now, we set an auth token
        auth_token = str(uuid.uuid4())
        self.client.is_token_valid = mock.MagicMock(return_value=True)
        self.client.set_auth_token(auth_token)

        # check that we don't allow submission of zero listens
        with self.assertRaises(errors.EmptyPayloadException):
            self.client.submit_multiple_listens([])

        # test that we check for validity of listen types
        with self.assertRaises(errors.UnknownListenTypeException):
            self.client._post_submit_listens([listen], 'unknown listen type')

        # test that we don't allow submission of multiple listens
        # with the "single" or "playing now" types
        with self.assertRaises(errors.TooManyListensException):
            self.client._post_submit_listens([listen, listen], 'single')

        with self.assertRaises(errors.TooManyListensException):
            self.client._post_submit_listens([listen, listen], 'playing_now')

        # test that we don't submit timestamps for playingnow listens
        with self.assertRaises(errors.ListenedAtInPlayingNowException):
            self.client.submit_playing_now(listen)

    def test_set_auth_token_invalid_token(self):
        self.client.is_token_valid = mock.MagicMock(return_value=False)

        with self.assertRaises(errors.InvalidAuthTokenException):
            self.client.set_auth_token(str(uuid.uuid4()))

    def test_set_auth_token_valid_token(self):
        self.client.is_token_valid = mock.MagicMock(return_value=True)
        auth_token = str(uuid.uuid4())
        self.client.set_auth_token(auth_token)
        self.assertEqual(auth_token, self.client._auth_token)

    @mock.patch('liblistenbrainz.client.requests.post')
    def test_post_api_exceptions(self, mock_requests_post):
        response = mock.MagicMock()
        response.json.return_value = {'code': 401, 'error': 'Unauthorized'}
        response.raise_for_status.side_effect = requests.HTTPError(response=response)
        mock_requests_post.return_value = response

        # set auth token
        self.client.is_token_valid = mock.MagicMock(return_value=True)
        auth_token = str(uuid.uuid4())
        self.client.set_auth_token(auth_token)

        # create a listen
        ts = int(time.time())
        listen = liblistenbrainz.Listen(
            track_name="Fade",
            artist_name="Kanye West",
            release_name="The Life of Pablo",
            listened_at=ts,
        )

        # assert that submitting it raises a ListenBrainzAPIException
        with self.assertRaises(errors.ListenBrainzAPIException):
            self.client.submit_single_listen(listen)

    @mock.patch('liblistenbrainz.client.requests.get')
    def test_get_api_exceptions(self, mock_requests_get):
        response = mock.MagicMock()
        response.json.return_value = {'code': 401, 'error': 'Unauthorized'}
        response.raise_for_status.side_effect = requests.HTTPError(response=response)
        mock_requests_get.return_value = response

        # assert that getting listens raises a ListenBrainzAPIException
        with self.assertRaises(errors.ListenBrainzAPIException):
            self.client.get_listens('iliekcomputers')


    @mock.patch('liblistenbrainz.client.requests.get')
    def test_get_user_recommendation_recordings(self, mock_requests_get):
        mock_requests_get.return_value = mock.MagicMock()
        with self.assertRaises(ValueError):
            self.client.get_user_recommendation_recordings("boo", "bad artist type")

        mock_requests_get.reset_mock()
        mock_requests_get.return_value = mock.MagicMock()
        self.client.get_user_recommendation_recordings("iliekcomputers", "top", offset=3, count=2)
        mock_requests_get.assert_called_once_with(
            'https://api.listenbrainz.org/1/cf/recommendation/user/iliekcomputers/recording',
            params={
                    'artist_type': 'top',
                    'count': 2,
                    'offset': 3
                   },
            headers={},
        )

        mock_requests_get.reset_mock()
        mock_requests_get.return_value = mock.MagicMock()
        self.client.get_user_recommendation_recordings("iliekcomputers", "similar", offset=3, count=2)
        mock_requests_get.assert_called_once_with(
            'https://api.listenbrainz.org/1/cf/recommendation/user/iliekcomputers/recording',
            params={
                    'artist_type': 'similar',
                    'count': 2,
                    'offset': 3
                   },
            headers={},
        )

    def test_get_user_listen_count(self):
        self.client._get = mock.MagicMock()

        test_response = {"payload":{"count":111487}}
        self.client._get.return_value = test_response
        returned_count = self.client.get_user_listen_count('iliekcomputers')
        self.client._get.assert_called_once_with('/1/user/iliekcomputers/listen-count')
        
        self.assertEqual(returned_count, test_response['payload']['count'])




# liblistenbrainz - A simple client library for ListenBrainz
# Copyright (C) 2020 Param Singh <iliekcomputers@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

LISTEN_TYPE_SINGLE = 'single'
LISTEN_TYPE_IMPORT = 'import'
LISTEN_TYPE_PLAYING_NOW = 'playing_now'

LISTEN_TYPES = (
    LISTEN_TYPE_SINGLE,
    LISTEN_TYPE_IMPORT,
    LISTEN_TYPE_PLAYING_NOW,
)

class Listen:
    def __init__(
        self,
        track_name,
        artist_name,
        listened_at=None,
        release_name=None,
        recording_mbid=None,
        artist_mbids=None,
        release_mbid=None,
        tags=None,
        release_group_mbid=None,
        work_mbids=None,
        tracknumber=None,
        spotify_id=None,
        listening_from=None,
        isrc=None,
        additional_info=None,
        username=None,
        recording_msid=None,
    ):
        """ Creates a Listen.

        Needs at least a track name and an artist name.

        :param track_name: the name of the track
        :type track_name: str
        :param artist_name: the name of the artist
        :type artist_name: str
        :param listened_at: the unix timestamp at which the user listened to this listen
        :type listened_at: int, optional
        :param release_name: the name of the MusicBrainz release the track is a part of
        :type release_name: str, optional
        :param recording_mbid: the MusicBrainz ID of this listen's recording
        :type recording_mbid: str, optional
        :param artist_mbids: the MusicBrainz IDs of this listen's artists
        :type artist_mbids: List[str], optional
        :param release_mbid: the MusicBrainz ID of this listen's release
        :type release_mbid: str, optional
        :param tags: a list of user defined tags for this recording, each listen can only have at most 50
            tags and each tag must be shorter than 64 characters.
        :type tags: List[str], optional
        :param release_group_mbid: A MusicBrainz Release Group ID of the release group this
            recording was played from.
        :type release_group_mbid: str, optional
        :param work_mbids: A list of MusicBrainz Work IDs that may be associated with this recording.
        :type work_mbids: List[str], optional
        :param tracknumber: The tracknumber of the recording. This first recording on a release is tracknumber 1.
        :type tracknumber: int, optional
        :param spotify_id: The Spotify track URL associated with this
            recording. e.g.: http://open.spotify.com/track/1rrgWMXGCGHru5bIRxGFV0
        :type spotify_id: str, optional
        :param listening_from: the source of the listen, for example: 'spotify' or 'vlc',
        :type listening_from: str, optional
        :param isrc: The ISRC code associated with the recording.
        :type isrc: str, optional
        :param additional_info: a dict containing any additional fields that should be submitted with the listen.
        :type additional_info: dict, optional
        :param username: the username of the user to whom this listen belongs
        :type username: str, optional
        :param recording_msid: the MSID of this listen's recording
        :type recording_msid: str, optional
        :return: A listen object with the passed properties
        :rtype: Listen
        """
        self.listened_at = listened_at
        self.track_name = track_name
        self.artist_name = artist_name
        self.release_name = release_name
        self.recording_mbid = recording_mbid
        self.artist_mbids = artist_mbids or []
        self.release_mbid = release_mbid
        self.tags = tags or []
        self.release_group_mbid = release_group_mbid or []
        self.work_mbids = work_mbids or []
        self.tracknumber = tracknumber
        self.spotify_id = spotify_id
        self.listening_from = listening_from
        self.isrc = isrc
        self.additional_info = additional_info or {}
        self.username = username
        self.recording_msid = recording_msid


    def _to_submit_payload(self):
        # create the additional_info dict first
        additional_info = self.additional_info
        if self.recording_mbid:
            additional_info['recording_mbid'] = self.recording_mbid
        if self.artist_mbids:
            additional_info['artist_mbids'] = self.artist_mbids
        if self.release_mbid:
            additional_info['release_mbid'] = self.release_mbid
        if self.tags:
            additional_info['tags'] = self.tags
        if self.release_group_mbid:
            additional_info['release_group_mbid'] = self.release_group_mbid
        if self.work_mbids:
            additional_info['work_mbids'] = self.work_mbids
        if self.tracknumber is not None:
            additional_info['tracknumber'] = self.tracknumber
        if self.spotify_id:
            additional_info['spotify_id'] = self.spotify_id
        if self.listening_from:
            additional_info['listening_from'] = self.listening_from
        if self.isrc:
            additional_info['isrc'] = self.isrc

        # create track_metadata now and put additional_info into it if it makes sense
        track_metadata = {
            'track_name': self.track_name,
            'artist_name': self.artist_name,
        }
        if self.release_name:
            track_metadata['release_name'] = self.release_name
        if additional_info:
            track_metadata['additional_info'] = additional_info

        # create final payload and put track metadata into it
        payload = {
            'track_metadata': track_metadata,
        }
        if self.listened_at is not None:
            payload['listened_at'] = self.listened_at

        return payload



import liblistenbrainz
import time

auth_token = input('Please enter your auth token: ')

listen = liblistenbrainz.Listen(
    track_name="Fade",
    artist_name="Kanye West",
    release_name="The Life of Pablo",
    listened_at=int(time.time()),
)

client = liblistenbrainz.ListenBrainz()
client.set_auth_token(auth_token)
response = client.submit_single_listen(listen)
assert response['status'] == 'ok'

listen_2 = liblistenbrainz.Listen(
    track_name="Contact",
    artist_name="Daft Punk",
    release_name="Random Access Memories",
    listened_at=int(time.time()),
)

listen_3 = liblistenbrainz.Listen(
    track_name="Get Lucky",
    artist_name="Daft Punk",
    release_name="Random Access Memories",
    listened_at=int(time.time()),
)

response = client.submit_multiple_listens([listen_2, listen_3])
assert response['status'] == 'ok'



# liblistenbrainz - A simple client library for ListenBrainz
# Copyright (C) 2020 Param Singh <iliekcomputers@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

class ListenBrainzException(Exception):
    pass

class ListenBrainzAPIException(ListenBrainzException):
    def __init__(self, status_code, message=None):
        super(ListenBrainzAPIException, self).__init__()
        self.status_code = status_code
        self.message = message


class AuthTokenRequiredException(ListenBrainzException):
    pass


class InvalidAuthTokenException(ListenBrainzException):
    pass


# Exceptions that can be thrown when submitting listens
class InvalidSubmitListensPayloadException(ListenBrainzException):
    pass


class EmptyPayloadException(InvalidSubmitListensPayloadException):
    pass


class UnknownListenTypeException(InvalidSubmitListensPayloadException):
    pass


class TooManyListensException(InvalidSubmitListensPayloadException):
    pass


class ListenedAtInPlayingNowException(InvalidSubmitListensPayloadException):
    pass



# liblistenbrainz - A simple client library for ListenBrainz
# Copyright (C) 2020 Param Singh <iliekcomputers@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from liblistenbrainz import errors
from liblistenbrainz.listen import Listen
from liblistenbrainz.listen import LISTEN_TYPES, LISTEN_TYPE_SINGLE, LISTEN_TYPE_IMPORT, LISTEN_TYPE_PLAYING_NOW

def _validate_submit_listens_payload(listen_type, listens):
    if not listens:
        raise errors.EmptyPayloadException("Can't submit empty list of listens")

    if listen_type not in LISTEN_TYPES:
        raise errors.UnknownListenTypeException(f"Invalid listen type: {str(listen_type)}")


    if listen_type != LISTEN_TYPE_IMPORT and len(listens) != 1:
        raise errors.TooManyListensException("Too many listens for listen type %s: %d" % (str(listen_type), len(listens)))

    if listen_type == LISTEN_TYPE_PLAYING_NOW and listens[0].listened_at is not None:
        raise errors.ListenedAtInPlayingNowException("There is a listened_at field in a listen meant to be sent as `playing_now`")


def _convert_api_payload_to_listen(data):
    track_metadata = data['track_metadata']
    additional_info = track_metadata.get('additional_info', {})
    return Listen(
        track_name=track_metadata['track_name'],
        artist_name=track_metadata['artist_name'],
        listened_at=data.get('listened_at'),
        release_name=track_metadata.get('release_name'),
        recording_mbid=additional_info.get('recording_mbid'),
        artist_mbids=additional_info.get('artist_mbids', []),
        release_mbid=additional_info.get('release_mbid'),
        tags=additional_info.get('tags', []),
        release_group_mbid=additional_info.get('release_group_mbid'),
        work_mbids=additional_info.get('work_mbids', []),
        tracknumber=additional_info.get('tracknumber'),
        spotify_id=additional_info.get('spotify_id'),
        listening_from=additional_info.get('listening_from'),
        isrc=additional_info.get('isrc'),
        additional_info=additional_info,
        username=data.get('username'),
        recording_msid=data.get('recording_msid'),
    )



# liblistenbrainz - A simple client library for ListenBrainz
# Copyright (C) 2020 Param Singh <iliekcomputers@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import json
import requests
import time

from datetime import datetime
from enum import Enum
from liblistenbrainz import errors
from liblistenbrainz.listen import LISTEN_TYPE_IMPORT, LISTEN_TYPE_PLAYING_NOW, LISTEN_TYPE_SINGLE
from liblistenbrainz.utils import _validate_submit_listens_payload, _convert_api_payload_to_listen
from urllib.parse import urljoin

STATS_SUPPORTED_TIME_RANGES = (
    'week',
    'month',
    'quarter',
    'half_yearly',
    'year',
    'all_time',
    'this_week',
    'this_month',
    'this_year'
)


API_BASE_URL = 'https://api.listenbrainz.org'

class ListenBrainz:

    def __init__(self):
        self._auth_token = None

        # initialize rate limit variables with None
        self._last_request_ts = None
        self.remaining_requests = None
        self.ratelimit_reset_in = None


    def _require_auth_token(self):
        if not self._auth_token:
            raise errors.AuthTokenRequiredException


    def _wait_until_rate_limit(self):
        # if we haven't made any request before this, return
        if self._last_request_ts is None:
            return

        # if we have available requests in this window, return
        if self.remaining_requests and self.remaining_requests > 0:
            return

        # if we don't have available requests and we know when the
        # window is reset, backoff until the window gets reset
        if self.ratelimit_reset_in is not None:
            reset_ts = self._last_request_ts + self.ratelimit_reset_in
            current_ts = int(time.time())
            if current_ts < reset_ts:
                time.sleep(reset_ts - current_ts)


    def _update_rate_limit_variables(self, response):
        self._last_request_ts = int(time.time())
        try:
            self.remaining_requests = int(response.headers.get('X-RateLimit-Remaining'))
        except (TypeError, ValueError):
            self.remaining_requests = None

        try:
            self.ratelimit_reset_in = int(response.headers.get('X-RateLimit-Reset-In'))
        except (TypeError, ValueError):
            self.ratelimit_reset_in = None


    def _get(self, endpoint, params=None, headers=None):
        if not params:
            params = {}
        if not headers:
            headers = {}
        if self._auth_token:
            headers['Authorization'] = f'Token {self._auth_token}'

        try:
            self._wait_until_rate_limit()
            response = requests.get(
                urljoin(API_BASE_URL, endpoint),
                params=params,
                headers=headers,
            )
            self._update_rate_limit_variables(response)
            response.raise_for_status()
        except requests.HTTPError as e:
            status_code = e.response.status_code

            # get message from the json in the response if possible
            try:
                message = e.response.json().get('error', '')
            except Exception:
                message = None
            raise errors.ListenBrainzAPIException(status_code=status_code, message=message) from e

        if response.status_code == 204:
            raise errors.ListenBrainzAPIException(status_code=204)
        return response.json()


    def _post(self, endpoint, data=None, headers=None):
        if not headers:
            headers = {}
        if self._auth_token:
            headers['Authorization'] = f'Token {self._auth_token}'
        try:
            self._wait_until_rate_limit()
            response = requests.post(
                urljoin(API_BASE_URL, endpoint),
                data=data,
                headers=headers,
            )
            self._update_rate_limit_variables(response)
            response.raise_for_status()
        except requests.HTTPError as e:
            status_code = e.response.status_code

            # get message from the json in the response if possible
            try:
                message = e.response.json().get('error', '')
            except Exception:
                message = None
            raise errors.ListenBrainzAPIException(status_code=status_code, message=message) from e

        return response.json()


    def _post_submit_listens(self, listens, listen_type):
        self._require_auth_token()
        _validate_submit_listens_payload(listen_type, listens)
        listen_payload = [listen._to_submit_payload() for listen in listens]
        submit_json = {
            'listen_type': listen_type,
            'payload': listen_payload
        }
        return self._post(
            '/1/submit-listens',
            data=json.dumps(submit_json),
        )


    def set_auth_token(self, auth_token, check_validity=True):
        """
        Give the client an auth_token to use for future requests.
        This is required if the client wishes to submit listens. Each user
        has a unique auth token and the auth token is used to identify the user
        whose data is being submitted.

        :param auth_token: auth token
        :type auth_token: str
        :param check_validity: specify whether we should check the validity
            of the auth token by making a request to ListenBrainz before setting it (defaults to True)
        :type check_validity: bool, optional
        :raises InvalidAuthTokenException: if ListenBrainz tells us that the token is invalid
        :raises ListenBrainzAPIException: if there is an error with the validity check API call
        """
        if not check_validity or self.is_token_valid(auth_token):
            self._auth_token = auth_token
        else:
            raise errors.InvalidAuthTokenException


    def submit_multiple_listens(self, listens):
        """ Submit a list of listens to ListenBrainz.

        Requires that the auth token for the user whose listens are being submitted has been set.

        :param listens: the list of listens to be submitted
        :type listens: List[liblistenbrainz.Listen]
        :raises ListenBrainzAPIException: if the ListenBrainz API returns a non 2xx return code
        :raises InvalidSubmitListensPayloadException: if the listens sent are invalid, see exception message for details
        """
        return self._post_submit_listens(listens, LISTEN_TYPE_IMPORT)


    def submit_single_listen(self, listen):
        """ Submit a single listen to ListenBrainz.

        Requires that the auth token for the user whose data is being submitted has been set.

        :param listen: the listen to be submitted
        :type listen: liblistenbrainz.Listen
        :raises ListenBrainzAPIException: if the ListenBrainz API returns a non 2xx return code
        :raises InvalidSubmitListensPayloadException: if the listen being sent is invalid, see exception message for details
        """
        return self._post_submit_listens([listen], LISTEN_TYPE_SINGLE)


    def submit_playing_now(self, listen):
        """ Submit a playing now notification to ListenBrainz.

        Requires that the auth token for the user whose data is being submitted has been set.

        :param listen: the listen to be submitted, the listen should NOT have a `listened_at` attribute
        :type listen: liblistenbrainz.Listen
        :raises ListenBrainzAPIException: if the ListenBrainz API returns a non 2xx return code
        :raises InvalidSubmitListensPayloadException: if the listen being sent is invalid, see exception message for details
        """
        return self._post_submit_listens([listen], LISTEN_TYPE_PLAYING_NOW)

    def submit_user_feedback(self, feedback, recording_mbid):
        """ Submit a feedback to Listenbrainz
            
        Requires that the auth token for the user whose data is being submitted has been set.

        :param feedback The type of feedback 1 = loved, -1 = hated, 0 = delete feedback if any
        :param recording_mbid The recording Musicbrainz Id of the track being anotated
        """
        data = {
            'score': feedback,
            'recording_mbid': recording_mbid,
        }
        headers = {
            'Content-Type': 'application/json',
        }
        return self._post(
            '/1/feedback/recording-feedback',
            data=json.dumps(data),
            headers=headers,
        )


    def delete_listen(self, listen):
        """ Delete a particular listen from a user’s listen history.

        The listen is not deleted immediately, but is scheduled for deletion, which usually happens shortly after the hour.

        Requires that the auth token for the user whose data is being submitted has been set.

        :param listen: the listen to be deleted. The listen must have a `listened_at` and `recording_msid` attribute
        :type listen: liblistenbrainz.Listen
        :raises ListenBrainzAPIException: if the ListenBrainz API returns a non 2xx return code
        :raises InvalidSubmitListensPayloadException: if the listen being sent is invalid, see exception message for details
        """
        data = {
            'listened_at': listen.listened_at,
            'recording_msid': listen.recording_msid,
        }
        headers = {
            'Content-Type': 'application/json',
        }
        return self._post(
            '/1/delete-listen',
            data=json.dumps(data),
            headers=headers,
        )


    def is_token_valid(self, token):
        """ Check if the specified ListenBrainz auth token is valid using the ``/1/validate-token`` endpoint.

        :param token: the auth token that needs to be checked for validity
        :type token: str
        :raises ListenBrainzAPIException: if the ListenBrainz API returns a non 2xx return code
        """
        data = self._get(
            '/1/validate-token',
            params={'token': token},
        )
        return data['valid']


    def get_playing_now(self, username):
        """ Get the listen being played right now for user `username`.

        :param username: the username of the user whose data is to be fetched
        :type username: str
        :return: A single listen if the user is playing something currently, else None
        :rtype: liblistenbrainz.Listen or None
        :raises ListenBrainzAPIException: if the ListenBrainz API returns a non 2xx return code
        """
        data = self._get('/1/user/{username}/playing-now'.format(username=username))
        listens = data['payload']['listens']
        if len(listens) > 0: # should never be greater than 1
            return _convert_api_payload_to_listen(listens[0])
        return None


    def get_listens(self, username, max_ts=None, min_ts=None, count=None):
        """ Get listens for user `username`

        If none of the optional arguments are given, this function will return the 25 most recent listens.
        The optional `max_ts` and `min_ts` UNIX epoch timestamps control at which point in time to start returning listens.
        You may specify max_ts or min_ts, but not both in one call.

        :param username: the username of the user whose data is to be fetched
        :type username: str
        :param max_ts: If you specify a max_ts timestamp, listens with listened_at less than (but not including) this value will be returned.
        :type max_ts: int, optional
        :param min_ts: If you specify a min_ts timestamp, listens with listened_at greater than (but not including) this value will be returned.
        :type min_ts: int, optional
        :param count: the number of listens to return. Defaults to 25, maximum is 100.
        :type count: int, optional
        :return: A list of listens for the user `username`
        :rtype: List[liblistenbrainz.Listen]
        :raises ListenBrainzAPIException: if the ListenBrainz API returns a non 2xx return code
        """
        params = {}
        if max_ts is not None:
            params['max_ts'] = max_ts
        if min_ts is not None:
            params['min_ts'] = min_ts
        if count is not None:
            params['count'] = count

        data = self._get(
            '/1/user/{username}/listens'.format(username=username),
            params=params,
        )
        listens = data['payload']['listens']
        return [_convert_api_payload_to_listen(listen_data) for listen_data in listens]

    def _get_user_entity(self, username, entity, count=25, offset=0, time_range='all_time'):
        if time_range not in STATS_SUPPORTED_TIME_RANGES:
            raise errors.ListenBrainzException(f"Invalid time range: {time_range}")

        params = {
            'count': count,
            'offset': offset,
            'range': time_range,
        }

        try:
            return self._get(f'/1/stats/user/{username}/{entity}', params=params)
        except errors.ListenBrainzAPIException as e:
            if e.status_code == 204:
                return None
            else:
                raise


    def get_user_artists(self, username, count=25, offset=0, time_range='all_time'):
        """ Get artists for user 'username', sorted in descending order of listen count.

        :param username: the username of the user whose artists are to be fetched.
        :type username: str

        :param count: the number of artists to fetch, defaults to 25, maximum is 100.
        :type count: int, optional

        :param offset: the number of artists to skip from the beginning, for pagination, defaults to 0.
        :type offset: int, optional

        :param time_range: the time range, can be 'all_time', 'month', 'week' or 'year'
        :type time_range: str

        :return: the artists listened to by the user in the time range with listen counts and other data in the same format as the API response
        :rtype: dict
        """
        return self._get_user_entity(username, 'artists', count, offset, time_range)


    def get_user_recordings(self, username, count=25, offset=0, time_range='all_time'):
        """ Get recordings for user 'username', sorted in descending order of listen count.

        :param username: the username of the user whose artists are to be fetched.
        :type username: str

        :param count: the number of recordings to fetch, defaults to 25, maximum is 100.
        :type count: int, optional

        :param offset: the number of recordings to skip from the beginning, for pagination, defaults to 0.
        :type offset: int, optional

        :param time_range: the time range, can be 'all_time', 'month', 'week' or 'year'
        :type time_range: str

        :return: the recordings listened to by the user in the time range with listen counts and other data, in the same format as the API response
        :rtype: dict
        """
        return self._get_user_entity(username, 'recordings', count, offset, time_range)


    def get_user_releases(self, username, count=25, offset=0, time_range='all_time'):
        """ Get releases for user 'username', sorted in descending order of listen count.

        :param username: the username of the user whose releases are to be fetched.
        :type username: str

        :param count: the number of releases to fetch, defaults to 25, maximum is 100.
        :type count: int, optional

        :param offset: the number of releases to skip from the beginning, for pagination, defaults to 0.
        :type offset: int, optional

        :param time_range: the time range, can be 'all_time', 'month', 'week' or 'year'
        :type time_range: str

        :return: the releases listened to by the user in the time range with listen counts and other data
        :rtype: dict
        """
        return self._get_user_entity(username, 'releases', count, offset, time_range)


    def get_user_recommendation_recordings(self, username, artist_type='top', count=25, offset=0):
        """ Get recommended recordings for a user.

        :param username: the username of the user whose recommended tracks are to be fetched.
        :type username: str

        :param artist_type: The type of filtering applied to the recommended tracks.
                            'top' for filtering by top artists or
                            'similar' for filtering by similar artists
                            'raw' for no filtering
        :type artist_type: str

        :param count: the number of recordings to fetch, defaults to 25, maximum is 100.
        :type count: int, optional

        :param offset: the number of releases to skip from the beginning, for pagination, defaults to 0.
        :type offset: int, optional

        :return: the recommended recordings as other data returned by the API
        :rtype: dict
        """

        if artist_type not in ('top', 'similar', 'raw'):
            raise ValueError("artist_type must be either top or similar or raw.")
        params = {
                    'artist_type': artist_type,
                    'count': count,
                    'offset': offset
                 }
        try:
            return self._get(f'/1/cf/recommendation/user/{username}/recording', params=params)

        except errors.ListenBrainzAPIException as e:
            if e.status_code == 204:
                return None
            else:
                raise

    def get_user_listen_count(self, username):
        """ Get total number of listens for user

        :param username: The username of the user whose listens are to be fetched
        :type username: str

        :return: Number of listens returned by the Listenbrainz API
        :rtype: int
        """
        try:
            return self._get(f'/1/user/{username}/listen-count')['payload']['count']
        except errors.ListenBrainzAPIException as e:
            if e.status_code == 204:
                return None
            else:
                raise

    def get_user_feedback(self, username, score, metadata, count=100, offset=0 ):
        """ Get feedback given by user
        
        :param username: The user to get the feedbacks from
        :type username: str
        :param count: Optional, number of feedback items to return. Default 100.
        :type count: int, optional
        :param offset:  Optional, number of feedback items to skip from the beginning, for pagination. Ex. An offset of 5 means the top 5 feedback will be skipped, defaults to 0.
        :type offset: int, optional
        :param score: Optional, If 1 then returns the loved recordings, if -1 returns hated recordings.
        :type score: int, optional
        :param metadata: Optional, boolean if this call should return the metadata for the feedback.
        :type metadata: bool, optional
        """
        params = {
                    'count': count,
                    'offset': offset,
                    'score': score,
                    'metadata': metadata,
                 }
        try:
            return self._get(f'/1/feedback/user/{username}/get-feedback', params=params)
        except errors.ListenBrainzAPIException as e:
            if e.status_code == 204:
                return None
            else:
                raise



import liblistenbrainz

auth_token = input('Please enter your auth token: ')

listen = liblistenbrainz.Listen(
    track_name="Fade",
    artist_name="Kanye West",
    release_name="The Life of Pablo",
)

client = liblistenbrainz.ListenBrainz()
client.set_auth_token(auth_token)
response = client.submit_playing_now(listen)
assert response['status'] == 'ok'




# liblistenbrainz - A simple client library for ListenBrainz
# Copyright (C) 2020 Param Singh <iliekcomputers@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import sys

if sys.version_info >= (3, 10):
    from importlib.metadata import version, PackageNotFoundError
else:
    # importlib.metadata's API changed in 3.10, so use a backport for versions less than this.
    from importlib_metadata import version, PackageNotFoundError

try:
    __version__ = version(__name__)
except PackageNotFoundError:
    # package is not installed?
    __version__ = "unknown"

from liblistenbrainz.client import ListenBrainz
from liblistenbrainz.listen import Listen
from liblistenbrainz.listen import LISTEN_TYPE_IMPORT, LISTEN_TYPE_PLAYING_NOW, LISTEN_TYPE_SINGLE
